#!/usr/bin/env bash

# Roger Firth's Cloak of Darkness implemented in pure Bash 4.
# Conversion by Felix Pleşoianu <https://felix.plesoianu.ro>.

# Made with the SHTAP game engine, a.k.a. the SHell Text Adventure Player.
# You can use SHTAP to make your own games; just replace the game data.
# If you do, please keep this notice so that other authors may benefit too.

# Code preamble: don't edit until the next comment.

typeset -A name description location map lock success failure
typeset -A before after dark
typeset -iA points visited
typeset -lA noun synonym
typeset -A verb

noun[hero]=me
description[hero]='As good-looking as ever.'

# End of preamble: add your own game data below.

TITLE='Cloak of Darkness'
HEADLINE='A basic IF demonstration'
AUTHOR='Roger Firth'
DATE='2003'
INSTRUCTIONS=''

MAX_SCORE=10

BANNER="Hurrying through the rainswept November night, you're glad to see the bright lights of the Opera House. It's surprising that there aren't more people about but, hey, what do you expect in a cheap demo game...?"

location[hero]=foyer

location[cloak]=hero
name[cloak]='velvet cloak'
description[cloak]='A handsome cloak, of velvet trimmed with satin, and slightly spattered with raindrops. Its blackness is so deep that it almost seems to suck light from the room.'
noun[cloak]='black velvet cloak satin'
points[cloak]=5

name[foyer]='Foyer of the Opera House'
description[foyer]='You are standing in a spacious hall, splendidly decorated in red and gold, with glittering chandeliers overhead. The entrance from the street is to the north, and there are doorways south and west.'

map[west,foyer]=cloakroom
map[south,foyer]=bar

map[north,foyer]=foyer
lock[north,foyer]=true
failure[north,foyer]="You've only just arrived, and besides, the weather outside seems to be getting worse."

map[out,foyer]=foyer
lock[out,foyer]=true
failure[out,foyer]="You've only just arrived, and besides, the weather outside seems to be getting worse."

name[cloakroom]=Cloakroom
description[cloakroom]='The walls of this small room were clearly once lined with hooks, though now only one remains. The exit is a door to the east.'

name[hook]='small brass hook'
noun[hook]='small brass hook'
description[hook]="It's just a small brass hook, screwed to the wall."
location[hook]=cloakroom
lock[hook]=true

after[examine,hook]=xhook
function xhook {
	if [ "${location[cloak]}" = hook ]; then
		echo "Your cloak is neatly hanging from it at the moment."
	fi
}

map[east,cloakroom]=foyer
map[out,cloakroom]=foyer

before[drop,cloak]=drop_cloak
function drop_cloak {
	if [ "${location[hero]}" = cloakroom ]; then
		score_object cloak
		true
	else
		echo "This isn't the best place to leave a smart cloak lying around."
		false
	fi
}

function hang {
	if [ "${location[$1]}" != hero ]; then
		echo "But you don't have that."
	elif [ "${location[hero]}" != "${location[hook]}" ]; then
		echo "There's nothing here to hang it on."
	else
		echo "You neatly hang up the ${name[$1]}."
		location[$1]=hook
		score_object "$1"
	fi
}

verb[hang]=hang

name[bar]='Foyer bar'
description[bar]="The bar, much rougher than you'd have guessed after the opulence of the foyer to the north, is completely empty. There seems to be some sort of message scrawled in the sawdust on the floor."

dark[bar]=is_dark
function is_dark {
	test "${location[cloak]}" = hero
}

name[message]='scrawled message'
noun[message]='scrawled sawdust message'
location[message]=bar
lock[message]=true
points[message]=5

before[examine,message]=message
function message {
	if is_dark; then
		echo "Inside of a dog it's too dark to read."
	elif [ "${visited[bar]}" -gt 2 ]; then
		echo "The message has been carelessly trampled, making it difficult to read. You can just distinguish the words..."
		end_game "You have lost"
	else
		score_object message
		echo "The message, neatly marked in the sawdust, reads..."
		end_game "You have won"
	fi
	false
}
verb[read]=examine

map[north,bar]=foyer

# Game engine starts here. You shouldn't need to edit below this line.
# SHTAP r1 beta, from 2018-06-18: just enough for a Cloak of Darkness.

typeset -iA score
typeset -i moves=0

synonym[n]=north
synonym[s]=south
synonym[e]=east
synonym[w]=west
synonym[u]=up
synonym[d]=down
synonym[ne]=northeast
synonym[se]=southeast
synonym[nw]=northwest
synonym[sw]=southwest

function examine {
	if [ -n "${description[$1]}" ]; then
		echo "${description[$1]}"
	else
		echo "You see nothing unexpected."
	fi
	moves+=1
}

verb[examine]=examine
verb[x]=examine

function look {
	local here=${location[hero]}

	echo "---"
	echo "${name[$here]}"
	echo "---"
	
	if dark_object "$here"; then
		if [ -n "${failure[$here]}" ]; then
			echo "${failure[$here]}"
		else
			echo "It's too dark to see much at all."
		fi
	else
		echo "${description[$here]}"
		if [ -n "${success[$here]}" ]; then
			echo
			echo "${success[$here]}"
		fi

		local inv=$(list_content $here)
		if [ -n "$inv" ]; then
			echo
			echo "Visible objects:"
			echo "$inv"
		fi
		
		score_object $here
	fi

	visited[$here]+=1
}

verb[look]=look
verb[l]=look

function inventory {
	echo
	local inv=$(list_content hero)
	if [ -n "$inv" ]; then
		echo 'You are carrying:'
		echo "$inv"
	else
		echo 'You are empty-handed.'
	fi
	moves+=1
}

verb[inventory]=inventory
verb[i]=inventory

function list_content {
	for i in ${!location[@]}; do
		if [ "${location[$i]}" = "$1" ]; then
			if dark_object "$i"; then
				continue
			elif [ "$i" = hero ]; then
				continue
			else
				echo " - ${name[$i]}"
			fi
		fi
	done
}

function dark_object {
	if [ -n "${dark[$1]}" ]; then
		${dark[$1]}
	else
		false
	fi
}

function score_object {
	if [ -z "${points[$1]}" ]; then
		return 1
	elif [ -z ${score[$1]} ]; then
		score[$1]=${points[$1]}
		if [ ${points[$1]} -eq 1 ]; then
			echo "(You score 1 point.)"
		else
			echo "(You score ${points[$1]} points.)"
		fi
		return 0
	fi
}

function show_score {
	local -i full_score=0

	for i in ${score[@]}; do
		full_score+=$i
	done

	echo -n "In $moves moves,"

	if [ $full_score -eq 1 ]; then
		echo -n " you've scored 1 point"
	else
		echo -n " you've scored $full_score points"
	fi

	if [ -n "$MAX_SCORE" ]; then
		echo -n " out of $MAX_SCORE possible"
	fi

	echo "."
}

verb[score]=show_score

function restart {
	local -l answer
	echo -n "Start over? (yes/no) "
	read answer
	if [ "${answer:0:1}" = "y" ]; then
		exec $0
	fi
}

verb[restart]=restart

function quit {
	local -l answer
	echo -n "Really quit? (yes/no) "
	read answer
	if [ "${answer:0:1}" = "y" ]; then
		exit
	fi
}

verb[quit]=quit

function take {
	if [ "${location[$1]}" = hero ]; then
		echo "But you already have it."
	elif item_locked "$1"; then
		if [ -n "${failure[$1]}" ]; then
			echo "${failure[$1]}"
		else
			echo "That's hardly portable."
		fi
	else
		location[$1]=hero
		
		moves+=1
		visited[$1]+=1
		score_object $1
		
		if [ -n "${success[$1]}" ]; then
			echo "${success[$1]}"
		else
			echo "Taken."
		fi
	fi
}

verb[take]=take
verb[get]=take

function item_locked {
	if [ -n "${lock[$1]}" ]; then
		${lock[$1]}
	else
		false
	fi
}

function drop {
	if [ "${location[$1]}" = hero ]; then
		location[$1]=${location[hero]}
		moves+=1
		echo "Dropped."
	else
		echo "But you don't have that."
	fi
}

verb[drop]=drop

function travel {
	local here=${location[hero]}
	if [ -z "${map[$1,$here]}" ]; then
		return 1
	elif exit_locked "$1" "$here"; then
		if [ -n "${failure[$1,$here]}" ]; then
			echo "${failure[$1,$here]}"
		else
			echo "You can't pass."
		fi
		return 0
	else
		if [ -n "${success[$1,$here]}" ]; then
			echo "${success[$1,$here]}"
			echo
		fi
		location[hero]=${map[$1,$here]}
		look
		moves+=1
		return 0
	fi
}

function exit_locked {
	if [ -n "${lock[$1,$2]}" ]; then
		${lock[$1,$2]}
	else
		false
	fi
}

function verbs {
	echo "This adventure understands the following verbs: ${!verb[*]}."
}

verb[verbs]=verbs

function end_game {
	echo
	echo "   *** $1 ***   "
	echo
	show_score
	echo
	select answer in quit restart continue; do
		if [ $answer = quit ]; then
			exit
		elif [ $answer = restart ]; then
			exec $0
		else
			break
		fi
	done
}

function lookup_word {
	if [ -z "$1" ]; then
		return 0
	elif [ -z "${synonym[$1]}" ]; then
		echo "$1"
		return 0
	else
		echo "${synonym[$1]}"
		return 0
	fi
}

function find_in {
	for i in ${!location[@]}; do
		if [ "${location[$i]}" = "$1" ]; then
			for j in ${noun[$i]}; do
				if [ "$j" = "$2" ]; then
					echo "$i"
					return 0
				fi
			done
		fi
	done
	return 1
}

function find_here {
	local here=${location[hero]}
	if find_in "$here" "$1"; then
		return 0
	else
		for i in ${!location[@]}; do
			if [ "${location[$i]}" = "$here" ]; then
				if find_in "$i" "$1"; then
					return 0
				fi
			fi
		done
	fi
}

function test_before {
	if [ -n "${before[$1,$2]}" ]; then
		${before[$1,$2]}
	else
		true
	fi
}

# Main loop

echo "$BANNER"
echo
echo "$TITLE"
echo "$HEADLINE"
echo "$AUTHOR, $DATE"
echo

look

echo
if [ -n "$INSTRUCTIONS" ]; then
	echo "$INSTRUCTIONS"
else
	echo '(Enter a verb and at most one noun. Type VERBS to see a list.)'
fi

typeset -l cmdverb cmdnoun

while true; do
	echo
	echo -n "> "
	read cmdverb cmdnoun
	echo "$cmdverb $cmdnoun"

	cmdverb=$(lookup_word $cmdverb)
	cmdnoun=$(lookup_word $cmdnoun)

	if [ -z "$cmdverb" ]; then
		continue
	elif [ -z "${verb[$cmdverb]}" ]; then
		travel $cmdverb
		if [ $? -gt 0 ]; then
			echo "I don't know how to $cmdverb."
		fi
	elif [ -z "$cmdnoun" ]; then
		${verb[$cmdverb]}
	else
		cmdverb=${verb[$cmdverb]}
		cmdobj=$(find_here "$cmdnoun")
		if [ $? -gt 0 ]; then
			echo "You can't see any $cmdnoun."
		elif test_before "$cmdverb" "$cmdobj"; then
			$cmdverb "$cmdobj"
			if [ -n "${after[$cmdverb,$cmdobj]}" ]; then
				${after[$cmdverb,$cmdobj]}
			fi
		fi
	fi
done
